#include <gnome.h>
#include <glade/glade.h>

#include <utillib.h>

#include "gdownman-marshal.h"

#include "servers.h"


GType           servers_get_type   ();
static void     servers_class_init (ServersClass *klass);
static void     servers_init       (Servers *servers);

static void     servers_create_model_and_view   (Servers *servers);
void            servers_format_max_connections  (GtkTreeViewColumn *tree_column,
                                                 GtkCellRenderer *cell,
                                                 GtkTreeModel *tree_model,
                                                 GtkTreeIter *iter,
                                                 gpointer data);

gboolean    servers_response_cb     (GtkWidget *widget,
                                     GdkEvent *event,
                                     gpointer data);

void    servers_selection_changed       (GtkTreeSelection *selection,
                                         gpointer data);
void    servers_properties_button_cb    (GtkButton *button, gpointer data);
gboolean    servers_properties_delete_cb    (GtkWidget *widget,
                                             GdkEvent *event,
                                             gpointer data);
void    servers_properties_response_cb  (GtkDialog *dialog, gint arg1,
                                         gpointer data);
void    servers_properties_max_conn_toggled (GtkToggleButton *togglebutton,
                                             gpointer data);

enum
{
	COLUMN_NAME,
	COLUMN_MAX_CONNECTIONS,
	COLUMN_CURRENT_CONNECTIONS,
	NUM_COLUMNS
};

enum {
    LAST_SIGNAL
};

static gint servers_signals[LAST_SIGNAL] = { 0 };

static gpointer parent_class = NULL;


GType
servers_get_type ()
{
    static GType new_type = 0;

    if (!new_type)
    {
        static const GTypeInfo new_info =
        {
            sizeof (ServersClass),
            (GBaseInitFunc) NULL,
            (GBaseFinalizeFunc) NULL,
            (GClassInitFunc) servers_class_init,
            NULL,           /* class_finalize */
            NULL,           /* class_data */
            sizeof (Servers),
            0,              /* n_preallocs */
            (GInstanceInitFunc) servers_init
        };

        new_type = g_type_register_static (G_TYPE_OBJECT,
                                           "Servers",
                                           &new_info,
                                           0);
    }

    return new_type;
}

static void
servers_class_init (ServersClass *klass)
{
    parent_class = g_type_class_peek_parent (klass);
}

static void
servers_init (Servers *servers)
{
    servers->active = FALSE;
    servers->fd = -1;
}

Servers *
servers_new (guint fd, GtkWindow *parent)
{
    Servers *servers = SERVERS (g_object_new (servers_get_type (), NULL));
    GladeXML *xml;

    /* load the dialog */
    xml = glade_xml_new ("servers.glade", NULL , NULL);

    if (!xml)
    {
        xml = glade_xml_new (GLADEDIR "/" "servers.glade", NULL ,
                             NULL);

        if (!xml)
        {
            g_warning ("The \"servers dialog\" can't be loaded.");
            return NULL;
        }
    }

    /* servers dialog */
    servers->dialog = glade_xml_get_widget (xml, "servers_dialog");
    servers->vbox = glade_xml_get_widget (xml, "vbox");

    gtk_window_set_transient_for (GTK_WINDOW (servers->dialog), parent);
    gtk_window_set_default_size (GTK_WINDOW (servers->dialog), 400, 300);
    gtk_window_set_destroy_with_parent (GTK_WINDOW (servers->dialog), TRUE);

    g_signal_connect (G_OBJECT (servers->dialog), "delete-event",
                      G_CALLBACK (servers_response_cb),
                      G_OBJECT (servers->dialog));

    g_signal_connect_swapped (G_OBJECT (servers->dialog), "response",
                              G_CALLBACK (gtk_widget_hide),
                              servers->dialog);

    servers->tree_view = GTK_TREE_VIEW (glade_xml_get_widget (xml,
                                                              "tree_view"));

    /* get the selection */
    servers->selection = gtk_tree_view_get_selection (
                                        GTK_TREE_VIEW (servers->tree_view));

    gtk_tree_selection_set_mode (servers->selection, GTK_SELECTION_SINGLE);

    g_signal_connect (G_OBJECT (servers->selection), "changed",
                      GTK_SIGNAL_FUNC (servers_selection_changed), servers);

    /* properties button to active the properties dialog */
    servers->properties_button = glade_xml_get_widget (xml,
                                                       "properties_button");
    gtk_widget_set_sensitive (servers->properties_button, FALSE);
    g_signal_connect (G_OBJECT (servers->properties_button), "clicked",
                      G_CALLBACK (servers_properties_button_cb),
                      G_OBJECT (servers));

    /* properties dialog and events */
    servers->properties_dialog  = glade_xml_get_widget (xml,
                                                        "properties_dialog");
    gtk_window_set_transient_for (GTK_WINDOW (servers->properties_dialog),
                                  GTK_WINDOW (servers->dialog));
    gtk_window_set_destroy_with_parent (GTK_WINDOW (servers->properties_dialog),
                                        TRUE);
    g_signal_connect (G_OBJECT (servers->properties_dialog), "delete-event",
                      G_CALLBACK (servers_properties_delete_cb),
                      G_OBJECT (servers));
    g_signal_connect (G_OBJECT (servers->properties_dialog), "response",
                      G_CALLBACK (servers_properties_response_cb),
                      servers);

    servers->max_conn_spin = glade_xml_get_widget (xml, "max_conn_spin");
    servers->max_conn_check = glade_xml_get_widget (xml, "max_conn_check");

    g_signal_connect (G_OBJECT (servers->max_conn_check), "toggled",
                      G_CALLBACK (servers_properties_max_conn_toggled),
                      G_OBJECT (servers));

    glade_xml_signal_autoconnect (xml);

    g_object_unref (G_OBJECT (xml));

    servers->fd = fd;

    servers_create_model_and_view (servers);

    return servers;
}

static void
servers_create_model_and_view (Servers *servers)
{
    GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

    /* create tree model */
    servers->model = GTK_TREE_MODEL (gtk_list_store_new (NUM_COLUMNS,
                                                         G_TYPE_STRING,
                                                         G_TYPE_INT,
                                                         G_TYPE_UINT));

    gtk_tree_view_set_model (servers->tree_view, servers->model);

    renderer = gtk_cell_renderer_text_new ();
    g_object_set (renderer, "xalign", 0.0, NULL);
    column = gtk_tree_view_column_new_with_attributes ("Server",
                                                       renderer,
                                                       "text",
                                                       COLUMN_NAME,
                                                       NULL);
    gtk_tree_view_column_set_sort_column_id (column, COLUMN_NAME);
    gtk_tree_view_append_column (servers->tree_view, column);

    renderer = gtk_cell_renderer_text_new ();
    g_object_set (renderer, "xalign", 1.0, NULL);
    column = gtk_tree_view_column_new_with_attributes ("Maximum",
                                                       renderer,
                                                       "text",
                                                       COLUMN_MAX_CONNECTIONS,
                                                       NULL);
    gtk_tree_view_column_set_cell_data_func (column, renderer,
                        (GtkTreeCellDataFunc) servers_format_max_connections,
                        GINT_TO_POINTER (COLUMN_MAX_CONNECTIONS),
                        NULL);
    gtk_tree_view_column_set_sort_column_id (column, COLUMN_MAX_CONNECTIONS);
    gtk_tree_view_append_column (servers->tree_view, column);

    renderer = gtk_cell_renderer_text_new ();
    g_object_set (renderer, "xalign", 1.0, NULL);
    column = gtk_tree_view_column_new_with_attributes ("Current",
                                                       renderer,
                                                       "text",
                                                       COLUMN_CURRENT_CONNECTIONS,
                                                       NULL);
    gtk_tree_view_column_set_sort_column_id (column, COLUMN_CURRENT_CONNECTIONS);
    gtk_tree_view_append_column (servers->tree_view, column);

    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(servers->tree_view), TRUE);
}

void
servers_format_max_connections (GtkTreeViewColumn *tree_column,
        GtkCellRenderer *cell, GtkTreeModel *tree_model, GtkTreeIter *iter,
        gpointer data)
{
	gint column = GPOINTER_TO_INT (data);
	GValue value = {0, };
	guint status;

	gtk_tree_model_get_value (tree_model, iter, column, &value);

	status = g_value_get_int (&value);

    if (status == -1)
        g_object_set (cell, "text", _("unlimited"), NULL);

	g_value_unset (&value);
}

void
servers_start (Servers *servers)
{
    if (!servers->active)
    {
        dm_command_server_events (servers->fd, 1);
        servers->active = TRUE;
    }

    gtk_window_present (GTK_WINDOW (servers->dialog));
}

void
servers_add (Servers *servers, gchar *name, gint max, guint current)
{
	GtkTreeIter iter;

    gtk_list_store_append (GTK_LIST_STORE (servers->model), &iter);
    gtk_list_store_set (GTK_LIST_STORE (servers->model), &iter,
                        COLUMN_NAME, name,
                        COLUMN_MAX_CONNECTIONS, max,
                        COLUMN_CURRENT_CONNECTIONS, current,
                        -1);
}

void
servers_max_connections (Servers *servers, gchar *name, gint max)
{
    GtkTreeIter iter;
    gchar *server_url;

    if (gtk_tree_model_get_iter_first (servers->model, &iter))
    {
        do {
            gtk_tree_model_get (servers->model, &iter,
                                COLUMN_NAME, &server_url,
                                -1);
            if (strcmp (name, server_url) == 0)
                break;
        } while (gtk_tree_model_iter_next (servers->model, &iter));
    }

    gtk_list_store_set (GTK_LIST_STORE (servers->model), &iter,
                        COLUMN_MAX_CONNECTIONS, max,
                        -1);
}

void
servers_current_connections (Servers *servers, gchar *name, guint current)
{
    GtkTreeIter iter;
    gchar *server_url;

    if (gtk_tree_model_get_iter_first (servers->model, &iter))
    {
        do {
            gtk_tree_model_get (servers->model, &iter,
                                COLUMN_NAME, &server_url,
                                -1);
            if (strcmp (name, server_url) == 0)
                break;
        } while (gtk_tree_model_iter_next (servers->model, &iter));
    }

    gtk_list_store_set (GTK_LIST_STORE (servers->model), &iter,
                        COLUMN_CURRENT_CONNECTIONS, current,
                        -1);
}

gboolean
servers_response_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
    gtk_widget_hide (widget);
    return TRUE;
}

void
servers_sensitive_set (Servers *servers, gboolean setting)
{
    gtk_widget_set_sensitive (servers->vbox, setting);
}

void
servers_selection_changed (GtkTreeSelection *selection, gpointer data)
{
    Servers *servers = SERVERS (data);
    gint selected_rows;

    selected_rows = gtk_tree_selection_count_selected_rows (selection);

    switch (selected_rows)
    {
        case 0:
            gtk_widget_set_sensitive (servers->properties_button, FALSE);
            break;

        case 1:
            gtk_widget_set_sensitive (servers->properties_button, TRUE);
            break;

        default:
            /* programming error */
            g_printerr ("selection error on servers dialog\n");
            break;
    }
}

void
servers_properties_button_cb (GtkButton *button, gpointer data)
{
    Servers *servers = SERVERS (data);
    GList * selected;
    GtkTreePath *path;
    GtkTreeIter iter;
    gint max_conn;

    selected = gtk_tree_selection_get_selected_rows (servers->selection, NULL);
    path = selected->data;
    if (gtk_tree_model_get_iter (servers->model, &iter, path) == 0)
    {
        g_printerr ("No iter for path found!\n");
        gtk_tree_path_free(path);
        return;
    } 

    gtk_tree_model_get (servers->model, &iter,
                        COLUMN_MAX_CONNECTIONS, &max_conn,
                        -1);

    switch (max_conn)
    {
        case -1:
            gtk_spin_button_set_value (GTK_SPIN_BUTTON (servers->max_conn_spin),
                                       0);
            gtk_widget_set_sensitive (servers->max_conn_spin, FALSE);
            gtk_toggle_button_set_active (
                                GTK_TOGGLE_BUTTON (servers->max_conn_check),
                                TRUE);
            break;

        default:
            gtk_spin_button_set_value (GTK_SPIN_BUTTON (servers->max_conn_spin),
                                       max_conn);
            gtk_widget_set_sensitive (servers->max_conn_spin, TRUE);
            gtk_toggle_button_set_active (
                                GTK_TOGGLE_BUTTON (servers->max_conn_check),
                                FALSE);
            break;
    }

    gtk_dialog_run (GTK_DIALOG (servers->properties_dialog));
}

gboolean
servers_properties_delete_cb (GtkWidget *widget, GdkEvent *event,
                                gpointer data)
{
    gtk_widget_hide (widget);
    return TRUE;
}

void
servers_properties_response_cb (GtkDialog *dialog, gint arg1, gpointer data)
{
    Servers *servers = SERVERS (data);
    GList * selected;
    GtkTreePath *path;
    GtkTreeIter iter;
    gchar *server_name;

    gtk_widget_hide (GTK_WIDGET (dialog));

    selected = gtk_tree_selection_get_selected_rows (servers->selection, NULL);
    path = selected->data;
    if (gtk_tree_model_get_iter (servers->model, &iter, path) == 0)
    {
        g_printerr ("No iter for path found!\n");
        gtk_tree_path_free(path);
        return;
    } 

    gtk_tree_model_get (servers->model, &iter,
                        COLUMN_NAME, &server_name,
                        -1);

    if (gtk_toggle_button_get_active (
                                GTK_TOGGLE_BUTTON(servers->max_conn_check)))
    {
        dm_command_server_max_connections (servers->fd, server_name, -1);
    }
    else
    {
        dm_command_server_max_connections (servers->fd, server_name,
                gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
                        (servers->max_conn_spin)));
    }
}

void
servers_properties_max_conn_toggled (GtkToggleButton *togglebutton, gpointer data)
{
    Servers *servers = SERVERS (data);
    gboolean checked;

    checked = gtk_toggle_button_get_active (
                                    GTK_TOGGLE_BUTTON(servers->max_conn_check));
    gtk_widget_set_sensitive (servers->max_conn_spin, !checked);
}
