#ifndef __CLIENT_H__
#define __CLIENT_H__

#include <glib-object.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define CLIENT(obj)           G_TYPE_CHECK_INSTANCE_CAST (obj, client_get_type (), Client)
#define CLIENT_CLASS(klass)   G_TYPE_CHECK_CLASS_CAST (klass, client_get_type (), ClientClass)
#define IS_CLIENT(obj)        G_TYPE_CHECK_INSTANCE_TYPE (obj, client_get_type ())


typedef struct _Client       Client;
typedef struct _ClientClass  ClientClass;

struct _Client
{
    GObject object;

    /*< private >*/
    guint id;
    GString *name;
    GIOChannel* socket;

    gint connected;

    GString *buffer;
    GString *buffer_in;
    GString *buffer_out;
    GString *buffer_read;

    GStaticMutex client_mutex;

    guint input_tag;
    guint output_tag;
    /* track if we can send to the client or must queue messages */
    gboolean output_busy;

    GQueue* output_messages;

    /* options */
    guint batch_status;
    guint batch_progress;
    guint events;
    guint events_projects;
    guint events_servers;
};

struct _ClientClass
{
    GObjectClass parent_class;

    void (* client_command_received)    (gchar *command);
    void (* client_disconnect)          (void);
};

GType           client_get_type       (void);
Client*         client_new            (gint fd);

void    client_send             (Client *client, gchar *buffer);

guint   client_events           (Client *client);
guint   client_events_projects  (Client *client);
guint   client_batch_status     (Client *client);
guint   client_batch_progress   (Client *client);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __CLIENT_H__ */
