#ifndef __LINELOG_H__
#define __LINELOG_H__

#include <glib-object.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define LINELOG(obj)            G_TYPE_CHECK_INSTANCE_CAST (obj,    \
                                            linelog_get_type (),    \
                                            Linelog)
#define LINELOG_CLASS(klass)    G_TYPE_CHECK_CLASS_CAST (klass,     \
                                            linelog_get_type (),    \
                                            LinelogClass)
#define IS_LINELOG(obj)         G_TYPE_CHECK_INSTANCE_TYPE (obj,    \
                                            linelog_get_type ())


typedef struct _Linelog       Linelog;
typedef struct _LinelogClass  LinelogClass;

/* The linelog object always adds elements to the start, keeps a reference
   to the last element on the list (first element added), and keeps a reference
   to the last requested element.
 */
struct _Linelog
{
    GObject object;

    /*< private >*/

    GList *begin;
    GList *end;
    /* Last message sent to clients */
    GList *last_seen;
    /* Last message written to disk */
    GList *last_written;

    /* path to log file */
    gchar *path;

    /* True if we must write lines to the file */
    gboolean disklog;

    GIOChannel *logfile;
};

struct _LinelogClass
{
    GObjectClass parent_class;
};

GType       linelog_get_type       (void);
Linelog*    linelog_new            ();

void        linelog_add     (Linelog *linelog, GString *line, GError **error);
GList*      linelog_get     (Linelog *linelog);
GList*      linelog_get_end (Linelog *linelog);

void        linelog_setfile (Linelog *linelog, gchar *path);
void        linelog_disklog (Linelog *linelog, gboolean setting,
                             GError **error);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __LINELOG_H__ */
