#ifndef __SERVERS_H__
#define __SERVERS_H__

#include <glib-object.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define SERVERS(obj)            G_TYPE_CHECK_INSTANCE_CAST (obj, servers_get_type (), Servers)
#define SERVERS_CLASS(klass)    G_TYPE_CHECK_CLASS_CAST (klass, servers_get_type (), ServersClass)
#define IS_SERVERS(obj)         G_TYPE_CHECK_INSTANCE_TYPE (obj, servers_get_type ())


typedef struct _Servers        Servers;
typedef struct _ServersClass   ServersClass;
typedef struct _Server         Server;

struct _Servers
{
    GObject object;

    /*< private >*/
    GHashTable *hash;
};

struct _ServersClass
{
    GObjectClass parent_class;

    /* emitted when a server is created */
    void (* servers_new)           (Server *server);

    /* emitted when a server changes a setting */
    void (* servers_changed)       (Server *server, gint setting);
};

struct _Server
{
    gchar *server_url;
    gint   max_connections;
    guint  current_connections;
};

GType       servers_get_type        (void);
Servers*    servers_new             (void);

gboolean    servers_connection_limit_reached    (Servers *servers,
                                                 const gchar *server_url);
void        servers_connection_start            (Servers *servers, 
                                                 const gchar *server_url);
void        servers_connection_end              (Servers *servers,
                                                 const gchar *server_url);

void        servers_server_new                  (Servers *servers,
                                                 Server *server);
void        servers_max_connections             (Servers *servers,
                                                 gchar *command);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __SERVERS_H__ */
